#include "bt.h"

#include <iostream>
#include <istream>
#include <strstream>
#include <fstream>

using namespace std;

char expression[] = "(((1+2)*3)--1)";
istrstream istristr(expression, strlen(expression));
istream& istr = istristr;

char GetNonSpaceCharacter(istream &istr) {

  char ret;
  istr >> ret;
  return ret;

}

char GetNonSpaceCharacter() {


  return GetNonSpaceCharacter(istr);

}

BT::BT() {

  parent = 0;
  char token = GetNonSpaceCharacter();
  bool isLeaf = false;
  if (token >= '0' && token <= '9') {
    opd = token-'0';
    isLeaf = true;
  }
  else if (token == '-') {
    token = GetNonSpaceCharacter();
    opd = '0'-token;
    isLeaf = true;
  }
  else if (token == '(') {
    isLeaf = false;
  }

  if (isLeaf) {
    left = right = 0;
    return;
  }

  left = new BT;

  token = GetNonSpaceCharacter();
  opr = token;

  right = new BT;

  token = GetNonSpaceCharacter();

  left->parent = this;
  right->parent = this;
  
}

ostream& operator<<(ostream &ostr, BT &bt) {

  if (bt.left == 0 && bt.right == 0) {
    return ostr << "[" << bt.opd << ", " << bt.x << "]";
  }
  else {
    return ostr << "(" << *bt.left 
      << "[" << bt.opr << ", " << bt.x << "]"
      << *bt.right << ")";
  }

}


int BT::EvaluateExpression() {

  if (left == 0 && right == 0)
    return opd;

  int leftval = left->EvaluateExpression();
  int rightval = right->EvaluateExpression();
  switch (opr) {
    case '+':
      return leftval + rightval;
    case '-':
      return leftval - rightval;
    case '*':
      return leftval * rightval;
    case '/':
      return leftval / rightval;
    default:
      cerr << "INTERNAL ERROR: unknown operator " << opr << endl;
      return 0;
  }

}

void BT::Draw() {

  // Step 1. find y coordinate as depth
  Sety(0);

  // Step 2. find raw x coordinate as in order rank
  int ior = 0;
  SetxInOrder(ior);

  // Step 3. finalize x coordinate post order centered above children
  FinalizeXCoordinate();

  // Step 4. create canvas in memory
  // Step 4.1 find bottom right corner
  float bottomf = -1.0f, rightf = -1.0f;
  FindCanvasBottomRightCorner(bottomf, rightf);
  int bottomi = (int)(bottomf)+1;
  int righti = (int)(rightf)+1;
  int scalef = 5;
  char *canvas = new char[bottomi*righti*scalef*scalef];
  
  for (int i = 0; i < bottomi*righti*scalef*scalef; i++)
    canvas[i] = ' ';

  // Step 5. dump tree on canvas
  DrawTree(canvas, righti, scalef);

  // Step 6. dump canvas to file or stdout
  ofstream ofs("mydbg/canvas.txt");
  for (int v = 0; v < bottomi*scalef; v++) {
    for (int u = 0; u < righti*scalef; u++) {
      ofs << canvas[v*righti*scalef+u];
    }
    ofs << endl;
  }
  ofs.close();


  // Step 7. release canvas
  delete canvas;

}

void BT::Sety(int level) {

  y = (float) level;
  if (left && right) {
    left->Sety(level+1);
    right->Sety(level+1);
  }

}


void BT::SetxInOrder(int &ior) {

  if (left)
    left->SetxInOrder(ior);

  x = (float) ior;
  ior++;

  if (right)
    right->SetxInOrder(ior);

}

void BT::FinalizeXCoordinate() {

  if (left)
    left->FinalizeXCoordinate();
  if (right)
    right->FinalizeXCoordinate();

  if (left && right)
    x = (left->x + right->x)/2.0f;

}

void BT::FindCanvasBottomRightCorner(float& bottomf, 
                                     float& rightf) {

  if (bottomf < y)
    bottomf = y;
  if (rightf < x)
    rightf = x;
  if (left)
    left->FindCanvasBottomRightCorner(bottomf, rightf);
  if (right)
    right->FindCanvasBottomRightCorner(bottomf, rightf);

}

void BT::DrawTree(char *canvas, int righti, int scalef) {

  if (left)
    left->DrawTree(canvas, righti, scalef);
  if (right)
    right->DrawTree(canvas, righti, scalef);

  int pixX = (int)(x*scalef);
  int pixY = (int)(y*scalef);
  if (left && right) {
    canvas[(pixY+2)*(scalef*righti)+pixX+2] = opr;
    DrawBox(canvas, righti, scalef, pixX, pixY, 'I');
  }
  else {
    if (opd < 0) {
      canvas[(pixY+2)*(scalef*righti)+pixX+1] = '-';
      canvas[(pixY+2)*(scalef*righti)+pixX+2] = -opd+'0';
    }
    else {
      canvas[(pixY+2)*(scalef*righti)+pixX+2] = opd+'0';
    }
    DrawBox(canvas, righti, scalef, pixX, pixY, 'E');
  }

}

void BT::DrawBox(char *canvas, int righti, 
  int scalef, int pixX, int pixY, char ink) {

  for (int i = 0; i < scalef; i++) { 
    canvas[(pixY)*(scalef*righti)+pixX+i] = ink;
    canvas[(pixY+4)*(scalef*righti)+pixX+i] = ink;
    canvas[(pixY+i)*(scalef*righti)+pixX] = ink;
    canvas[(pixY+i)*(scalef*righti)+pixX+4] = ink;
  }

}