#include "dlli.h"
#include <stdio.h>
#include <iostream>

using namespace std;

DLLINode::DLLINode(TiffImage *_tiffImage) :
  prev(0), next(0), tiffImage(_tiffImage) {
  
}

DLLI::DLLI() {

  header = new DLLINode(0);
  trailer = new DLLINode(0);
  header->next = trailer;
  trailer->prev = header;
  nodesN = 0;

}

void DLLI::InsertFirst(TiffImage *tiffImage) {

  InsertAfter(header, tiffImage);

}

void DLLI::InsertLast(TiffImage *tiffImage) {

  InsertAfter(trailer->prev, tiffImage);

}

void DLLI::InsertAfter(DLLINode *afterThis, TiffImage *tiffImage) {

  DLLINode *newNode = new DLLINode(tiffImage);

  newNode->prev = afterThis;
  newNode->next = afterThis->next;
  afterThis->next->prev = newNode;
  afterThis->next = newNode;

  nodesN++;

}

void DLLI::SaveAllImages() {

  SaveAllImages(0, header->next);

}

void DLLI::SaveAllImages(int imageIndex, 
                         DLLINode *curr) {

  if (curr == trailer)
    return;

  char fileName[100];
  sprintf_s(fileName, 100, "images/image%04d.tif", imageIndex);
  curr->tiffImage->Save(fileName);

  SaveAllImages(imageIndex+1, curr->next);

}

void DLLI::DeleteAllNodes(DLLINode *curr) {

  if (curr == header)
    return;

  DLLINode *nextToDelete = curr->prev;
  delete curr;
  DeleteAllNodes(nextToDelete);

}

DLLI::~DLLI() {

  cerr << "~DLLI() called" << endl;

  delete header;

}

DLLINode::~DLLINode() {

  cerr << "~DLLINode() called" << endl;

  if (tiffImage)
    delete tiffImage;

  if (next)
    delete next;
  
}
