#include "tiffImage.h"
#include <tiffio.h>
#include <fstream>
#include <iostream>

using namespace std;

void TiffImage::Resize(int neww, int newh, unsigned int color) {

  if (pixels)
    delete []pixels;

  w = neww;
  h = newh;
  pixels = new unsigned int[w*h];

  Set(color);

}

void TiffImage::Set(unsigned int color) {

  for (int i = 0; i < w*h; i++)
    pixels[i] = color;

}

void TiffImage::Init() {
  w = h = 0;
  pixels = 0;
}

TiffImage::TiffImage(int _w, int _h, unsigned int color) {

  w = _w;
  h = _h;
  pixels = new unsigned int[w*h];
  for (int i = 0; i < w*h; i++) {
    pixels[i] = color;
  }

}

TiffImage::TiffImage(char *tiffFileName) {

  Init();

  // make sure the file can be open
  ifstream tst(tiffFileName);
  if (tst.fail()) {
    cerr << "INFO: cannot open: " << tiffFileName << endl;
    return;
  }
  tst.close();

  TIFF* tif = TIFFOpen(tiffFileName, "r");

  if (tif) {

    cerr << "INFO: started loading: " << tiffFileName << endl;

    TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &w);
    TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &h);
    pixels = new unsigned int[w*h];

    if (!TIFFReadRGBAImage(tif, w, h, (uint32*) pixels, 0)) {
      delete pixels;
      Init();
      return;
    }
    for (int row = 0; row < h/2; row++) {
      for (int u = 0; u < w; u++) {
        unsigned int tmp = pixels[row*w+u];
        pixels[row*w+u] = pixels[(h-1-row)*w+u];
        pixels[(h-1-row)*w+u] = tmp;
      }
    }
  }

  TIFFClose(tif);

  cerr << "INFO: done loading: " << tiffFileName << endl;
  return;

}

void TiffImage::Save(char *fname) {
  
  TIFF* out = TIFFOpen(fname, "w");
  if (out == 0) {
    cerr << "ERROR: cannot open fname" << endl;
    return;
  }

  int sampleperpixel = 4;
  
  TIFFSetField(out, TIFFTAG_IMAGEWIDTH, w);
  TIFFSetField(out, TIFFTAG_IMAGELENGTH, h); 
  TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, sampleperpixel);
  TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, 8);
  TIFFSetField(out, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);
  
  TIFFSetField(out, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
  TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
  TIFFSetField(out, TIFFTAG_SOFTWARE, "Tiff writer");
  
  // We set the strip size of the file to be size of one row of pixels
  TIFFSetField(out, TIFFTAG_ROWSPERSTRIP, TIFFDefaultStripSize(out, w*sampleperpixel));
  
  // Now writing image to the file one strip at a time
  for (int row = 0; row < h; row++) {
    
    if (TIFFWriteScanline(out, &pixels[row*w], row) < 0)
      break;
  }
  
  TIFFClose(out);
  
}

TiffImage::~TiffImage() {

  cerr << "~TiffImage() called" << endl;
  if (pixels) {
    delete []pixels;
  }

}
